local ipAddrValidator = {}

ipAddrValidator.Type = {ERROR = 0, IPv4 = 1, IPv4_withMask = 2, IPv6 = 3, IPv6_withMask = 4, STRING = 5}

function ipAddrValidator.getIPType(ip)
    if type(ip) ~= "string" then
        return ipAddrValidator.Type.ERROR
    end
    
    -- check for IPv4 format
    local chunks = {ip:match("^(%d+)%.(%d+)%.(%d+)%.(%d+)(.*)")}
    if chunks ~= nil and #chunks == 5 then
        for i, v in ipairs(chunks) do
            if i < 5 and tonumber(v) > 255 then
                return ipAddrValidator.Type.STRING
            end
        end
        
        if chunks[5] == '' then
            return ipAddrValidator.Type.IPv4
        else
            local mask = string.match(chunks[5], "/(%d+)")
            if mask then
                mask = tonumber(mask)
                if mask >= 0 and mask <= 32 then
                    return ipAddrValidator.Type.IPv4_withMask
                end
            end
        end
        return ipAddrValidator.Type.STRING
    end
    
    -- check for IPv6 format
    local chunks = {}
    local istart, iend = ip:find("[a-fA-F0-9]*:")
    
    if istart == nil or istart > 1 then
        return ipAddrValidator.Type.STRING
    end
    
    while (istart ~= nil and istart < iend) do
        local chunk = ip:sub(istart, iend)
        if chunk and chunk:find(':') == 1 then
            chunk = chunk:sub(2, #chunk)
        end
        if chunk and chunk:find(':') == #chunk then
            chunk = chunk:sub(1, #chunk-1)
        end        
        table.insert(chunks, chunk)
        istart, iend = ip:find(":?[a-fA-F0-9]*:?", iend + 1)
    end    
    
    local mask = ip:sub(iend + 1, #ip)

    if #chunks < 9 then
        for i, v in ipairs(chunks) do
            if #v > 0 and tonumber(v, 16) > 65535 then
                return ipAddrValidator.Type.STRING
            end
        end
        
        if mask == '' then
            return ipAddrValidator.Type.IPv6
        else
            local maskNr = string.match(mask, "/(%d+)")
            if maskNr then
                maskNr = tonumber(maskNr)
                if maskNr >= 0 and maskNr <= 128 then
                    return ipAddrValidator.Type.IPv6_withMask
                end
            end
        end
    end
    
    return ipAddrValidator.Type.STRING
end

return ipAddrValidator
