local errType = require 'errType'
local utils = require 'utils'

local Certificate_Read = {}

function Certificate_Read.invoke(...)
    
    local stdout, err, errMsg = utils.shell({'openssl', 'x509', '-in', '/rex/rexcore/rexcore.cer', '-text', '-noout'})
    if err ~= nil then
        return nil, err, errMsg
    end
    
    local lines = utils.split(stdout, '\n')
    local result = {}
    for k, line in pairs(lines) do
        if result.issuer == nil then
            local issuerLine = string.match(line, 'Issuer:(.*)')
            if issuerLine then
                result.issuer = {}
                local params = utils.split(issuerLine, ',')
                for k, param in pairs(params) do
                    local key, value = string.match(param, '(%a+)=(.+)')
                    if key then
                        result.issuer[key] = utils.trim(value)
                    end
                end
            end
        end
        if result.subject == nil then
            local subjectLine = string.match(line, 'Subject:(.*)')
            if subjectLine then
                result.subject = {}
                local params = utils.split(subjectLine, ',')
                for k, param in pairs(params) do
                    local key, value = string.match(param, '(%a+)=(.+)')
                    if key then
                        result.subject[key] = utils.trim(value)
                    end
                end
            end
        end
        if result.validUntil == nil then
            result.validUntil = string.match(line, 'Not After%s*:%s*(.*)')
        end
        if result.validFrom == nil then
            result.validFrom = string.match(line, 'Not Before%s*:%s*(.*)')
        end
    end
    
    local stdout, err, errMsg = utils.shell({'openssl', 'x509', '-in', '/rex/rexcore/rexcore.cer', '-fingerprint', '-sha1', '-noout'})
    if err ~= nil then
        return nil, err, errMsg
    end
        
    result.sha1 = string.match(stdout, 'SHA1 Fingerprint%s*=%s*(.*)')
    if result.sha1 then
        result.sha1 = utils.trim(result.sha1)
    end
    
    local stdout, err, errMsg = utils.shell({'openssl', 'x509', '-in', '/rex/rexcore/rexcore.cer', '-fingerprint', '-sha256', '-noout'})
    if err ~= nil then
        return nil, err, errMsg
    end
    
    result.sha256 = string.match(stdout, 'SHA256 Fingerprint%s*=%s*(.*)')
    if result.sha256 then
        result.sha256 = utils.trim(result.sha256)
    end
    
    return result
end

return Certificate_Read
