local errType = require 'errType'
local utils = require 'utils'
local move = require 'FileSystem_Move'

local Certificate_Write = {}

function Certificate_Write.invoke(args)
    local key
    local cert
    local rexKeyPath = '/rex/rexcore/rexcore.key'
    local rexCrtPath = '/rex/rexcore/rexcore.cer'
    
    if args ~= nil then
        if utils.isArray(args) then
            if #args >= 1 then
                key = args[1]
            end
            if #args >= 2 then
                cert = args[2]
            end
        else
            key = args.key
            cert = args.cert
        end
    end
    
    if key ~= nil and type(key) ~= 'string' then
        return nil, errType.codes.InvalidParams, 'Invalid parameter \'key\'.'
    end
    if cert ~= nil and type(cert) ~= 'string' then
        return nil, errType.codes.InvalidParams, 'Invalid parameter \'cert\'.'
    end
    if cert == nil and key == nil then
        return nil, errType.codes.InvalidParams, 'No parameters given.'
    end
    
    -- store keys in temp
    local keyPath = ''
    if key then
        keyPath = '/tmp/rexcore.key'
        local res, err, errMsg = utils.writeToFile(keyPath, key)
        if res ~= nil then
            return nil, err, errMsg .. '\nFailed to save temporary file.'
        end
    else
        keyPath = rexKeyPath
    end
    
    local crtPath = ''
    if key then
        crtPath = '/tmp/rexcore.cer'
        local res, err, errMsg = utils.writeToFile(crtPath, cert)
        if res ~= nil then
            return nil, err, errMsg .. '\nFailed to save temporary file.'
        end
    else
        crtPath = rexCrtPath
    end
    
    -- check compatibility
    local stdout, err, errMsg = utils.shell({'openssl', 'rsa', '-in', keyPath, '-modulus', '-noout'})
    if err ~= nil then
        return nil, err, errMsg .. '\n Private key is not valid.'
    end
    local keyModulus = utils.trim(stdout)
    
    local stdout, err, errMsg = utils.shell({'openssl', 'x509', '-in', crtPath, '-modulus', '-noout'})
    if err ~= nil then
        return nil, err, errMsg .. '\n Certificate is not valid.'
    end
    local crtModulus = utils.trim(res.stdout)
    
    if keyModulus ~= crtModulus then
        return nil, errType.codes.InvalidParams, 'Private key and certificate do not match.'
    end
    
    -- update keys
    if key then
        local args = {}
        args.oldPath = keyPath
        args.newPath = rexKeyPath
        local res, err, errMsg = move.invoke(args)
        if err ~= nil then
            return nil, err, errMsg
        end
    end
    if cert then
        local args = {}
        args.oldPath = crtPath
        args.newPath = rexCrtPath
        local res, err, errMsg = move.invoke(args)
        if err ~= nil then
            return nil, err, errMsg
        end
    end
end

return Certificate_Write
