local errType = require 'errType'
local utils = require 'utils'

local FileSystem_List = {}

function FileSystem_List.invoke(args)
    local path
    
    if args ~= nil then
        if utils.isArray(args) then
            if #args >= 1 then
                path = args[1]
            end
        else
            path = args.path
        end
    end
    
    if path == nil or type(path) ~= 'string' then
        return nil, errType.codes.InvalidParams, 'Failed to read parameter \'path\'.'
    end
    
    local stdout, err, errMsg = utils.shell({'ls', '-l1a', '--time-style=+%Y-%m-%d %H:%M:%S', path})
    if err ~= nil then
        return nil, err, errMsg
    end
    
    local result = {}
    
    local lines = utils.split(stdout, '\n')
    for key, line in pairs(lines) do
        local permissions, numberOfLinks, owner, group, size, modifiedDate, modifiedTime, name
        = string.match(line, '([%w|-]+)%s+(%d+)%s+(%w+)%s+(%w+)%s+(%d+)%s+([%w|-]+)%s+([%w|:]+)%s+(.+)%s*')
        
        if name ~= nil then
            local item = {}
            if string.sub(permissions, 1, 1) == 'd' then
                item.directory = true
            end
            item.permissions = string.sub(permissions, 2)
            item.numberOfLinks = numberOfLinks
            item.owner = owner
            item.group = group
            item.size = tonumber(size)
            item.modified = modifiedDate .. ' ' .. modifiedTime
            item.name = name
            
            table.insert(result, item)
        end
    end
    
    return result
end

return FileSystem_List
