local errType = require 'errType'
local exec = require 'resty.exec'
local utils = require 'utils'
local Network_GetState = require 'Network_GetState'
local removeMethod = require 'FileSystem_Remove'

local Network_RemoveConfig = {}

function Network_RemoveConfig.invoke(args)
    local ifaceName
    
    if args ~= nil then
        if utils.isArray(args) then
            if #args >= 1 then
                ifaceName = args[1]
            end
        else
            ifaceName = args.interface
        end
    end
    
    if ifaceName == nil or type(ifaceName) ~= 'string' then
        return nil, errType.codes.InvalidParams, 'Failed to read parameter \'interface\'.'
    end
    
    local interfaces = Network_GetState.invoke()
    if interfaces == nil then
        return nil, errType.codes.InternalError, 'Failed to query interfaces.'
    end
    
    local interface
    for i, iface in ipairs(interfaces) do
        if iface.name == ifaceName then
            interface = iface
        end
    end
    if interface == nil then
        return nil, errType.codes.InvalidParams, 'Interface \'' .. interfaceName .. '\' not found.'
    end
    
    if interface.managed then
        local fileName = '/etc/systemd/network/' .. interface.name .. '.network'
        if utils.fileExists(fileName) then
            local ret, err, msg = removeMethod.invoke({fileName})
            if err ~= nil then
                return nil, err, 'Failed to remove file ' .. fileName .. '.\n' .. msg
            end
        end
        if interface.managedByHwAddr then
            local fileName = '/etc/systemd/network/' .. Network_GetState.hwaddr2name(interface.hwaddr) .. '.network'
            if utils.fileExists(fileName) then
                local ret, err, msg = removeMethod.invoke({fileName})
                if err ~= nil then
                    return nil, err, 'Failed to remove file ' .. fileName .. '.\n' .. msg
                end
            end
        end
    end
end

return Network_RemoveConfig
