local utils = require 'utils'

local RexCore_GetState = {}

function RexCore_GetState.invoke(...)
    
    local stdout, err, errMsg, exitcode = utils.shell({'systemctl', 'status', 'rexcore'})
    
    -- exitcode == 3 means that service is stopped
    if err ~= nil and exitcode ~= 3 then
        return nil, err, errMsg
    end
    
    local resp = {}    
    local blocks = utils.split(stdout, '\n\n')
    stats = blocks[1]
    local logs
    if #blocks > 1 then
        logs = blocks[2]
    end
    
    if stats ~= nil then
        local lines = utils.split(stats, '\n')
        stats = {}
        for k, line in pairs(lines) do
            if stats.loaded == nil then
                local loaded, path, enabled = string.match(line, 'Loaded:%s+(%w+)%s+%(([^;]+)%s*;%s*([^;]+)%s*;.*%)')
                stats.loaded = loaded
                stats.path = path
                stats.enabled = enabled
            end
            
            if stats.active == nil then
                local active, since = string.match(line, 'Active:%s+(.+)%s+since(.+)')
                stats.active = active
                stats.since = since
            end
        end
        
        if next(stats) ~= nil then
            resp.status = stats
        end
    end
    
    if logs ~= nil then
        local lines = utils.split(logs, '\n')
        logs = {}
        
        for k, v in pairs(lines) do
            if v ~= '' then
                table.insert(logs, v)
            end
        end
        
        resp.logs = logs
    end
    
    return resp
end

return RexCore_GetState
