local utils = require 'utils'

local TimeDate_GetState = {}

function TimeDate_GetState.invoke(...)
    local stdout, err, errMsg = utils.shell({'cat', '/proc/uptime'})
    if err ~= nil then
        return nil, err, errMsg
    end
    local uptime = {}
    local up, idle = string.match(stdout, '(%d+%.%d+)%s+(%d+%.%d+)')
    if up ~= nil then
        uptime.up = tonumber(up)
    end
    if idle ~= nil then
        uptime.idle = tonumber(idle)
    end
    
    local filesystemLocked = false
    if utils.isFilesystemReadOnly() then
        filesystemLocked = true
        utils.unlockFilesystem()
    end
    
    if filesystemLocked then
        utils.lockFilesystem()
    end
    local stdout, err, errMsg = utils.shell({'timedatectl', 'status'})
    if err ~= nil then
        return nil, err, errMsg
    end
    
    local lines = utils.split(stdout, '\n')
    for key, line in pairs(lines) do
        local param, value = string.match(line, '%s*([%w|%s]+):(.+)%s*')
        if param ~= nil and value ~= nil then
            param = utils.trim(param)
            value = utils.trim(value)
            
            if (param == 'Local time') then
                uptime.localTime = utils.trim(string.match(value, '%w*([%d|%s|:|-]+)'))
            end
            
            if (param == 'Universal time') then
                uptime.universalTime = utils.trim(string.match(value, '%w*([%d|%s|:|-]+)'))
            end
            
            if (param == 'RTC time') then
                uptime.rtc = utils.trim(string.match(value, '%w*([%d|%s|:|-]+)'))
            end
            
            if (param == 'Time zone') then
                uptime.zone = utils.trim(string.match(value, '([%w|/]+)%s'))
            end
            
            if (param == 'Network time on') then
                uptime.ntpEnabled = utils.trim(value)
                if uptime.ntpEnabled == 'yes' then
                    uptime.ntpEnabled = true
                else
                    uptime.ntpEnabled = false
                end
            end
            
            if (param == 'NTP synchronized') then
                uptime.ntpSynchronized = utils.trim(value)
                if uptime.ntpSynchronized == 'yes' then
                    uptime.ntpSynchronized = true
                else
                    uptime.ntpSynchronized = false
                end
            end
            
            if (param == 'RTC in local TZ') then
                uptime.rtcInLocal = utils.trim(value)
                if uptime.rtcInLocal == 'yes' then
                    uptime.rtcInLocal = true
                else
                    uptime.rtcInLocal = false
                end
            end
        end
    end
    
    local stdout, err, errMsg = utils.shell({'cat', '/etc/systemd/timesyncd.conf'})
    if err ~= nil then
        return nil, err, errMsg
    end
    
    local lines = utils.split(stdout, '\n')
    for key, line in pairs(lines) do
        local ntpServers = string.match(line, 'Servers%s*=%s*(.*)')
        if ntpServers ~= nil and ntpServers ~= '' then
            uptime.ntpServers = utils.split(ntpServers, ' ')
        end
        ntpServers = string.match(line, 'NTP%s*=%s*(.*)')
        if ntpServers ~= nil and ntpServers ~= '' then
            uptime.ntpServers = utils.split(ntpServers, ' ')
        end
    end
    
    return uptime
end

return TimeDate_GetState
