local errType = require 'errType'
local exec = require 'resty.exec'
local utils = require 'utils'

local alternativeExecutivesService = {}

function alternativeExecutivesService.execHasHMI(exec, hmis)
    if exec.Path ~= nil then
        local folder = string.match(exec.Path, 'exec%-alt/(.+)/exec%.rex')
        if folder ~= nil then
            for key, hmi in pairs(hmis) do
                if folder == string.match(hmi, 'exec%-alt/(.+)/hmi') then
                    return true
                end
            end
        end
    end
    return false
end

function alternativeExecutivesService.service(...)
    local prog = exec.new('/tmp/exec.sock')
    prog.argv = {'bash', '-c', 'RexExecTool /rex/rexcore/exec-alt/**/exec.rex'}
    local res, err = prog()
    if err ~= nil or res.stderr ~= '' then
        local stderr = ''
        if res and res.stderr then
            stderr = res.stderr
        end
        return nil, errType.codes.ServerError,
            utils.generateErrorMsg('Failed to execute shell command', err, stderr)
    end
    
    local executives = {}
    local blocks = utils.split(res.stdout, '\n\n')
    for key, block in pairs(blocks) do
        local lines = utils.split(block, '\n')
        local exec = {}
        for key, line in pairs(lines) do
            local param, value = string.match(line, '(.+):(.+)')
            if param == nil then
                param = ''
            end
            if value == nil then
                value = ''
            end
            
            param = utils.trim(param)
            value = utils.trim(value)
            
            exec[param] = value
        end
        table.insert(executives, exec)
    end
    
    prog.argv = {'bash', '-c', 'ls -Q /rex/rexcore/exec-alt/**/hmi.rex'}
    local res, err = prog()
    if err ~= nil or res.stderr ~= '' then
        local stderr = ''
        if res and res.stderr then
            stderr = res.stderr
        end
        return nil, errType.codes.ServerError,
            utils.generateErrorMsg('Failed to execute shell command', err, stderr)
    end
    
    local hmis = utils.split(res.stdout, '"')
    for key, exec in pairs(executives) do
        exec.hmi = alternativeExecutivesService.execHasHMI(exec, hmis)
    end
    
    
    return executives
end

return alternativeExecutivesService
