local cjson = require 'cjson.safe'
local upload = require "resty.upload"

local function getFilePath(header)
    if #header >= 3 and header[1] == 'Content-Disposition' then
        local content = header[2]
        local path = string.match(content, 'name=\"(.*)\";')
        return path
    end
    return nil
end

local chunk_size = 4096
local form = upload:new(chunk_size)
local file
while true do
    local typ, res, err = form:read()
    
    if not typ then
        ngx.say("failed to read: ", err)
        return
    end
    
    if typ == "header" then
        local file_name = getFilePath(res)
        if file_name then
            file = io.open(file_name, "w+")
            if not file then
                ngx.log(ngx.ERR, 'Failed to open file for writing: ' .. file_name)
                ngx.status = ngx.HTTP_INTERNAL_SERVER_ERROR
                ngx.say("Failed to open file for writing: ", file_name)
                ngx.exit(ngx.HTTP_OK)
                return
            end
        end
    
    elseif typ == "body" then
        if file then
            file:write(res)
        end
    
    elseif typ == "part_end" then
        file:close()
        file = nil
    
    elseif typ == "eof" then
        break
    end
end