local errType = require 'errType'
local exec = require 'resty.exec'

local utils = {}

function utils.isArray(t)
    
    if type(t) ~= 'table' then
        return false
    end
    
    local count = 0
    for k, v in pairs(t) do
        if type(k) ~= 'number' then
            return false
        else
            count = count + 1
        end
    end
    --all keys are numerical. now let's see if they are sequential and start with 1
    for i = 1, count do
        if not t[i] and type(t[i]) ~= 'nil' then
            return false
        end
    end
    return true
end

function utils.isInt(n)
    return type(n) == 'number' and n == math.floor(n)
end

function utils.split(str, inSplitPattern)
    local outResults = {}
    local theStart = 1
    local theSplitStart, theSplitEnd = string.find(str, inSplitPattern, theStart)
    while theSplitStart do
        table.insert(outResults, string.sub(str, theStart, theSplitStart - 1))
        theStart = theSplitEnd + 1
        theSplitStart, theSplitEnd = string.find(str, inSplitPattern, theStart)
    end
    table.insert(outResults, string.sub(str, theStart))
    return outResults
end

function utils.trim(s)
    if s then
        return s:match "^%s*(.-)%s*$"
    end
    return s
end

function string.startsWith(String,Start)
   return string.sub(String,1,string.len(Start))==Start
end

function utils.endsWith(String,End)
   return End=='' or string.sub(String,-string.len(End))==End
end

function utils.generateErrorMsg(msg, err, exitcode, stderr, unknown)
    local buf = msg
    if err ~= nil and err ~= '' then
        buf = buf .. '\n | error:\n' .. tostring(err)
    end
    if exitcode ~= nil and exitcode ~= 0 then
        buf = buf .. '\n | exitcode:\n' .. tostring(exitcode)
    end
    if stderr ~= nil and stderr ~= '' then
        buf = buf .. '\n | stderr:\n' .. tostring(stderr)
    end
    if unknown ~= nil and unknown ~= '' then
        buf = buf .. '\n | unknown:\n' .. tostring(unknown)
    end
    return buf
end

function utils.writeToFile(path, content)
    local prog = exec.new('/tmp/exec.sock')
    prog.argv = {'tee', path .. '.tmp'}
    prog.stdin = tostring(content)
    
    local res, err = prog()
    if err ~= nil or res.stderr ~= '' then
        return false
    end
    
    prog.argv = {'mv', path .. '.tmp', path}
    
    local res, err = prog()
    if err ~= nil or res.stderr ~= '' then
        return false, errType.codes.ServerError,
            utils.generateErrorMsg('Failed to write to file \'' .. path .. '\'', err, stderr)
    end
    
    return true
end

function utils.removeFile(path)
    local prog = exec.new('/tmp/exec.sock')
    prog.argv = {'rm', path}
    local res, err = prog()
    if err ~= nil or res.stderr ~= '' then
        return false
    end
end

function utils.fileExists(path)
    local f = io.open(path, "r")
    if f ~= nil then
        io.close(f) return true
    else
        return false
    end
end

function utils.isFilesystemReadOnly()
    local prog = exec.new('/tmp/exec.sock')
    prog.argv = {'mount'}
    local res, err = prog()
    if err ~= nil or res.stderr ~= '' then
        return false
    end
    
    local lines = utils.split(res.stdout, '\n')
    for key, line in pairs(lines) do
        local setting = line:match('on%s+/%s+.*%((%a%a)')
        if setting then
            if setting == 'ro' then
                return true
            else
                return false
            end
        end
    end
    return false
end

function utils.lockFilesystem()
    local prog = exec.new('/tmp/exec.sock')
    prog.argv = {'mount', '-o', 'remount,ro', '/'}
    local res, err = prog()
    if err ~= nil or res.stderr ~= '' then
        return false
    end
    return true
end

function utils.unlockFilesystem()
    local prog = exec.new('/tmp/exec.sock')
    prog.argv = {'mount', '-o', 'remount,rw', '/'}
    local res, err = prog()
    if err ~= nil or res.stderr ~= '' then
        return false
    end
    return true
end

function utils.reverseTable(tbl)
    for i = 1, math.floor(#tbl / 2) do
        local tmp = tbl[i]
        tbl[i] = tbl[#tbl - i + 1]
        tbl[#tbl - i + 1] = tmp
    end
end

function utils.shell(argv, stdin)
    local prog = exec.new('/tmp/exec.sock')
    prog.argv = argv
    local res, err = prog()
    local stdout = ''
    if res ~= nil then
        stdout = res.stdout
    end
    if err ~= nil then
        return stdout, errType.codes.ServerError,
            utils.generateErrorMsg('Failed to execute a shell command.', err)
    end
    if res == nil then
        return stdout, errType.codes.ServerError,
            utils.generateErrorMsg('Failed to execute a shell command. Sockexec encountered an error.')
    end
    if res.unknown ~= nil and res.unknown ~= '' then
        ngx.log(ngx.ALERT, 'Shell command responds with an unknown output. Report bug to the sockexec developer. Output: ' .. res.unknown)

        -- unknown output usually contains stdout -> do not throw error, log error and try to continue instead
        res.stdout = res.unknown
        -- return stdout, errType.codes.ServerError,
        --     utils.generateErrorMsg('Failed to execute a shell command. Shell command responds with an unknown output.', err, res.exitcode, res.stderr, res.unknown), res.exitcode
    end
    if res.exitcode ~= 0 then
        return stdout, errType.codes.ServerError,
            utils.generateErrorMsg('Failed to execute a shell command. Command exits with error.', err, res.exitcode, res.stderr, res.unknown), res.exitcode
    end
    
    return stdout
end

return utils
